@@ -1,36 +1,36 @@
 /*
  * (C) Copyright 1996-2016 ECMWF.
- * 
+ *
  * This software is licensed under the terms of the Apache Licence Version 2.0
- * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
- * In applying this licence, ECMWF does not waive the privileges and immunities 
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ * In applying this licence, ECMWF does not waive the privileges and immunities
  * granted to it by virtue of its status as an intergovernmental organisation nor
  * does it submit to any jurisdiction.
  */
 
 /*! \file NetcdfOrcaInterpretor.h
     \brief Implementation of the Template class NetcdfOrcaInterpretor.
-    
+
     Magics Team - ECMWF 2004
-    
+
     Started: Tue 17-Feb-2004
-    
+
     Changes:
-    
+
 */
 
 #include "NetcdfOrcaInterpretor.h"
 #include "Factory.h"
-#include "Netcdf.h"
+#include "NetcdfData.h"
 #include <limits>
 
 using namespace magics;
 
-NetcdfOrcaInterpretor::NetcdfOrcaInterpretor() 
+NetcdfOrcaInterpretor::NetcdfOrcaInterpretor()
 {}
 
 
-NetcdfOrcaInterpretor::~NetcdfOrcaInterpretor() 
+NetcdfOrcaInterpretor::~NetcdfOrcaInterpretor()
 {}
 
 #include <boost/geometry/geometry.hpp>
@@ -43,7 +43,7 @@ bool NetcdfOrcaInterpretor::interpretAsMatrix(Matrix** data)
 
 	Netcdf netcdf(path_, dimension_method_);
 	NetVariable var = netcdf.getVariable(longitude_);
-	vector<long> dims;
+	vector<size_t> dims;
 	var.getDimensions(dims);
 
 
@@ -243,7 +243,7 @@ bool NetcdfOrcaInterpretor::interpretAsMatrix(Matrix** data)
 
 bool NetcdfOrcaInterpretor::interpretAsPoints(PointsList& points)
 {
-	
+
 	// later!
 
 	// get the data ...
@@ -261,31 +261,31 @@ bool NetcdfOrcaInterpretor::interpretAsPoints(PointsList& points)
 		netcdf.get(longitude_, lonm, first, last);
 		netcdf.get(latitude_,  latm, first, last);
 		netcdf.get(field_, data, first, last);
-		
+
         vector<double>::iterator lat = latm.begin();
         vector<double>::iterator lon = lonm.begin();
         vector<double>::iterator val = data.begin();
-        
+
 		while (lat != latm.end() ) {
             double value = *val;
-           if (std::isnan(value) ) 
+           if (std::isnan(value) )
                 value = missing;
 
-            if ( value  != missing) { 
+            if ( value  != missing) {
                 value = (value * scaling_) + offset_;
 		  	    points.push_back(new UserPoint(*lon, *lat, value));
             }
 			++lat;
 			++lon;
 			++val;
-
 		  }
 	}
-   
+
 	catch (MagicsException& e)
 	{
 		MagLog::error() << e << "\n";
 	}
+	return true;
 }
 
 void NetcdfOrcaInterpretor::customisedPoints(const Transformation& transformation, const std::set<string>&, CustomisedPointsList& out, int thinning)
@@ -303,8 +303,8 @@ void NetcdfOrcaInterpretor::customisedPoints(const Transformation& transformatio
 
 			netcdf.get(longitude_, longitudes, first, last);
 			netcdf.get(latitude_,  latitudes, first, last);
-			netcdf.get(u_component_, x_component, first, last);
-			netcdf.get(v_component_, y_component, first, last);
+			netcdf.get(x_component_, x_component, first, last);
+			netcdf.get(y_component_, y_component, first, last);
 
 			for ( int ind = 0; ind < latitudes.size(); ind += thinning) {
 				CustomisedPoint* point = new CustomisedPoint();
@@ -324,14 +324,46 @@ void NetcdfOrcaInterpretor::customisedPoints(const Transformation& transformatio
 
 /*!
  Class information are given to the output-stream.
-*/		
+*/
 void NetcdfOrcaInterpretor::print(ostream& out)  const
 {
 	out << "NetcdfOrcaInterpretor[";
 	NetcdfInterpretor::print(out);
-	NetcdfOrcaInterpretorAttributes::print(out);
+
 	out << "]";
 }
 
-static SimpleObjectMaker<NetcdfOrcaInterpretor, NetcdfInterpretor> netcdf_geovalues_interpretor("orca");
 
+NetcdfInterpretor* NetcdfOrcaInterpretor::guess(const NetcdfInterpretor& from)
+{
+	if ( from.field_.empty() &&  (from.x_component_.empty() || from.y_component_.empty() ) ) 
+		return 0;
+	
+	Netcdf netcdf(from.path_, from.dimension_method_);
+	
+	string variable = from.field_;
+
+// get the attribute coordinates 
+
+
+	
+	string coordinates = netcdf.getVariable(variable).getAttribute("coordinates", string(""));
+	string latlon("lat lon");
+	coordinates = coordinates.substr(0, latlon.size());
+	if ( coordinates == "lat lon" ) {
+		cout << "NetCOMPLEX NETCDF DETECTED" << endl;
+		NetcdfOrcaInterpretor* interpretor = new NetcdfOrcaInterpretor();
+		
+		interpretor->NetcdfInterpretor::copy(from);
+		interpretor->latitude_ = "lat";
+		interpretor->longitude_ = "lon";
+		interpretor->time_variable_ = netcdf.detect(variable, "time");
+		interpretor->level_variable_ = netcdf.detect(variable, "level");
+		interpretor->number_variable_ = netcdf.detect(variable, "number");
+		return interpretor;
+	}
+	return 0;
+}
+
+
+static SimpleObjectMaker<NetcdfOrcaInterpretor, NetcdfInterpretor> netcdf_geovalues_interpretor("orca");
